%-------------------------------------------------------
% Compute value and policy functions for a one-state
% variable model with inflation zones adapted to the
% 2-equation model in Orphanides and Wieland (EER 2000)
%
% us:  US-CBO annual data 1960-98 
% 12:  1 state variable and 2 shocks 
% gh:  Gauss-Hermite Quadrature 
% xx:  pre-grid search + golden section
%
% Note: to run must first choose "shocks" and "settings"
%       (see readme.txt)
%
% USES: val12ghxxinit, val12ghxxnext, myfmin, myinterp1q
%--------------------------------------------------------

% check settings
if     ~(multflag==1 | multflag==0)
    error('choose multflag=1 or 0')
elseif ~(poliflag==1 | poliflag==0)
    error('choose poliflag=1 or 0')
elseif ~(philflag==1 | philflag==0)
    error('choose philflag=1 or 0')
elseif ~(prefflag==1 | prefflag==0)
    error('choose prefflag=1 or 0')
elseif  (philflag==0 & prefflag==0)
    error('philflag==0 & prefflag==0 is not implemented')
elseif ~(striflag==1 | striflag==0)
    error('choose striflag=1 or 0')
elseif ~(shocflag==2 | shocflag==1 | shocflag==0 | shocflag==-1)
    error('choose shocflag=2 or 1 or 0 or -1')
end

% choose parameter values:
% linear Phillips curve
alpha=0.31;
% zone-linear Phillips curve
zeta=2.9;
alphaz=0.60;
% IS curve
rho=0.63; 
ksi=0.23;
rbar=2.7826; % constant=ksi*rbar=0.64
% loss function
omega=0.5; 
beta=0.9;
pitarg=0;
zetapr=2;
% variance of shocks
if shocflag==-1 % low variance case
    sigmap=0.05;
    sigmay=0.05;
else % normal variance case
    sigmap=(0.99/2)^2;
    sigmay=(1.80/2)^2;  
end

% case of strict inflation targeting
if striflag==1
    omega=1;
end

% save parameters
parameter=[alpha,zeta,alphaz,rho,ksi,rbar,omega,beta,pitarg,zetapr,sigmap,sigmay];

% load sets of nodes and weights for Gauss-Hermite Quadrature 
load gh10.dat;
load gh15.dat;
load gh30.dat;
load gh60.dat;
rno10=gh10(:,1);
wei10=gh10(:,2);
rno15=gh15(:,1);
wei15=gh15(:,2);
rno30=gh30(:,1);
wei30=gh30(:,2);
rno60=gh60(:,1);
wei60=gh60(:,2);
 
% choose one set of nodes and weights
rno=rno30;
wei=wei30;
 
% 2 normal shocks with mean 0 and variance sigma or sigmay
psh=sqrt(2.0).*rno.*sqrt(sigmap) + 0;
ysh=sqrt(2.0).*rno.*sqrt(sigmay) + 0;
% price and demand shocks
if shocflag==2 | -1
    [PSH,YSH] = ndgrid(psh,ysh);
% price shocks only
elseif shocflag==1
    PSH=psh;
    YSH=0;
% without uncertainty
elseif shocflag==0
    PSH=0;
    YSH=0;
end 
weip=wei; 
weiy=wei; 

% grid for single state variable (initial inflation) 
pdenselimits=[-10,10];
plimits=[-30,30];
pstep=0.1;
pdensegrid=pdenselimits(1):pstep:pdenselimits(2);
[junk,npdense]=size(pdensegrid);
npi=npdense+10;
pilag(1:5)=[plimits(1) -25 -20 -15 -12];
pilag(6:npdense+5)=pdensegrid;
pilag(npdense+6:npdense+10)=[12 15 20 25 plimits(2)];
pilag=pilag'; 

% grid for single control variable (intended output gap) 
xdenselimits=[-10,10];
xlimits=[-100,100];
xstep=0.5;
xdensegrid=xdenselimits(1):xstep:xdenselimits(2);
[junk,nxdense]=size(xdensegrid);
nx=nxdense+4;
xx(1:2)=[xlimits(1) -30];
xx(3:nxdense+2)=xdensegrid;
xx(nxdense+3:nxdense+4)=[30 xlimits(2)];
xx=xx'; 

% screen output
disp ' ';
disp 'Computing value and optimal policy...';
disp ' ';

% initialize value and policy function
hinit=zeros(npi,1);
vinit=zeros(npi,1);
vall=zeros(nx,npi);

% solve one-period problem (valinit)
% iterate over the p grid
for ip=1:1:npi
    p=pilag(ip);
    % pre-search by grid-search...     
    for ix=1:1:nx;
        vall(ix,ip)=val12ghxxinit(xx(ix),parameter,p,PSH,YSH,weip,weiy,flags);      
    end         
    [minvall,minxx]=min(vall(:,ip)); 
    xxstart=xx(minxx);
    % ...use minimum value as starting value for golden section search
    % minimize 
    hinitmin=myfmin('val12ghxxinit',xlimits(1),xlimits(2),xxstart,[],parameter,p,PSH,YSH,weip,weiy,flags); 
    hinit(ip,1)=hinitmin;
    vinit(ip,1)=val12ghxxinit(hinitmin,parameter,p,PSH,YSH,weip,weiy,flags);
end

% copy used in value iteration and integration step
hpp=hinit;
vpp=vinit;
% save final approximations for plotting one-period problem 
hfin=hinit;
vfin=vinit;

% copy used in policy iteration 
vpnext=vinit;
 
% maximal number of value iterations + 1
nj=25;

% to save approximations from each iteration
hprox=zeros(npi,nj);
vprox=zeros(npi,nj);
% first column, saved initial, one-period value and policy
hprox(:,1)=hinit;
vprox(:,1)=vinit;

% solve multi-period problem (valnext) ?
if multflag==1

% value iterations
valcheck=1;
valprecision=sqrt(eps);
polprecision=sqrt(eps);
j=1;

while  valcheck > valprecision;
    j=j+1; 

    % screen output
    disp ('   val'); 
    disp (j-1)

    % iterate over the p grid
    for ip=1:1:npi
        p=pilag(ip);
        % pre-search by grid-search...
        % integrate
        for ix=1:1:nx;
          vall(ix,ip)=val12ghxxnext(xx(ix),parameter,p,pilag,vpp,plimits,PSH,YSH,weip,weiy,flags);      
        end         
        [minvall,minxx]=min(vall(:,ip)); 
        xxstart=xx(minxx);
        % ...use minimum value as starting value for golden section search
        % minimize and integrate    
        xmin=myfmin('val12ghxxnext',xlimits(1),xlimits(2),xxstart,[],parameter,p,pilag,vpp,plimits,PSH,YSH,weip,weiy,flags);
        femin=val12ghxxnext(xmin,parameter,p,pilag,vpp,plimits,PSH,YSH,weip,weiy,flags);
        vprox(ip,j)=femin;
        hprox(ip,j)=xmin; 
    end
    vchk=abs(((beta).*(vprox(1:npi,j:j)-vprox(1:npi,j-1:j-1)))./((1-beta).*vprox(1:npi,j:j)));
    valcheck=max(vchk);     
    vpp=vprox(1:npi,j);
    hpp=hprox(1:npi,j);
    jfin=j;
   
    % end when maximum number of value iterations accomplished
    if j == nj
        valcheck=0;
    end
    
    % check if policy iterations needed
    if valcheck > valprecision;
        
        % number of policy iterations 
        nk=100; 
        polcheck=1;
        k=1;
 
        % policy iterations wanted ?
        if poliflag==1
             
            while  polcheck > polprecision;   
   	            k=k+1;  
        
                % screen output
                disp ('   val   pol'); 
                disp ([j-1   k-1]);
        
                % iterate over the p grid    
                for ip=1:1:npi 
                    p=pilag(ip); 
                    % integrate    
                    fenext=val12ghxxnext(hprox(ip,j),parameter,p,pilag,vpp,plimits,PSH,YSH,weip,weiy,flags);
                    vpnext(ip,1)=fenext;      
                end
                vpchk=abs(((beta).*(vpnext(1:npi,1)-vpp(1:npi,1)))./((1-beta).*vpnext(1:npi,1)));
                polcheck=max(vpchk); 
                vpp=vpnext;
                vprox(1:npi,j)=vpnext;     
         
                % end when maximum number of value iterations accomplished
                if k == nk
                    polcheck=0;
                end
            
            % end while loop for policy iterations 
            end 
        % end if policy iterations wanted
        end 
    % end check if policy iterations needed
    end
% end while loop for value iterations    
end
 
% save approximations from each iteration
hsave=hprox(:,1:jfin); 
vsave=vprox(:,1:jfin);
% save final approximations for plotting
hfin=hprox(:,jfin);
vfin=vprox(:,jfin);

% end if solving multi-period problem
end